from django.core.exceptions import ValidationError
from django.core.files import File

import pytest

from benevalibre.accounts.models import User

from .factories import UserFactory


@pytest.mark.django_db
class TestUserManager:
    def test_create_user(self):
        user = User.objects.create_user('mail@example.org')
        assert set(User.objects.exclude(pseudo='instance')) == set([user])

    def test_create_superuser(self):
        user = User.objects.create_superuser('mail@example.org', 'pass')
        assert user.is_superuser is True
        assert set(User.objects.exclude(pseudo='instance')) == set([user])


class TestUser:
    def test_name_format(self):
        user = User(email="mail@example.org")
        assert str(user) == "mail@example.org"

        user.last_name = "Dupond"
        assert user.short_name == "Dupond"

        user.first_name = "Camille"
        assert user.short_name == "Camille"
        assert user.full_name == "Camille Dupond"
        assert str(user) == "Camille Dupond"

    def test_pseudo_format(self):
        user = User(email="mail@example.org", pseudo="Pseudo")
        assert user.short_name == "Pseudo"
        assert user.full_name == "Pseudo"

        user.last_name = "Dupond"
        assert user.short_name == "Dupond"

        user.first_name = "Camille"
        assert user.short_name == "Camille"
        assert user.full_name == "Camille Dupond (Pseudo)"

    def test_clean_name(self):
        user = User(email="mail@example.org")

        with pytest.raises(ValidationError) as excinfo:
            user.clean()
        assert excinfo.value.code == 'name_required'

        user.pseudo = "Pseudo"
        user.clean()

    def test_clean_email(self):
        user = UserFactory.build(email="Mail@Example.Org")
        user.clean()
        # cf. django.contrib.auth.base_user.BaseUserManager.normalize_email
        assert user.email == "Mail@example.org"

    def test_email_user(self, mailoutbox):
        user = UserFactory.build()
        user.email_user('sujet', 'body')
        assert len(mailoutbox) == 1

    def test_is_staff(self):
        user = User(is_superuser=False)
        assert user.is_staff is False
        user.is_superuser = True
        assert user.is_staff is True

    @pytest.mark.django_db
    def test_add_avatar(self):
        user = UserFactory.build(email="Mail@example.Org")
        with open('benevalibre/utils/tests/images/flower.jpg', 'rb') as avatar:
            user.avatar = File(avatar)
            user.save()
        assert user.avatar
        assert user.avatar.thumbnail
        # a second time for testing update
        with open('benevalibre/utils/tests/images/flower.jpg', 'rb') as avatar:
            user.avatar = File(avatar)
            user.save()
        assert user.avatar
        assert user.avatar.thumbnail
