from datetime import date

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q

import reversion

from benevalibre.accounts.models import User


class BenevaloQuerySet(models.QuerySet):
    def inactive(self):
        return self.filter(is_active=False)

    def active(self):
        return self.filter(is_active=True)

    def by_user(self, user):
        if user.is_anonymous:
            return self.none()
        return self.filter(user=user)

    def viewable_by(self, user):
        # En l'état un⋅e utilisat⋅eur⋅rice ne voit des autres que ce qu'il⋅elle
        # peut gérer.
        owned = self.by_user(user)
        return owned | self.manageable_by(user)

    def manageable_by(self, user):
        if user.is_anonymous:
            return self.none()
        elif user.is_superuser:
            return self.all()
        else:
            # Tous les benevalos visibles d'un⋅e utilisat⋅eur⋅rice, c'est tous
            # les benevalo des associations liées aux engagements liés à un
            # role dont il⋅elle a la capacité can_manage_benevalos. Les siens
            # sont exclus s'ils ne sont pas liés à un rôle gestionnaire
            permitted_engagements = user.engagement_set.filter(
                Q(role__manage_benevalos=True)
                | Q(role__manage_association=True)
            )
            permitted = self.filter(
                association__engagement__in=permitted_engagements
            )
            return permitted


BenevaloManager = models.Manager.from_queryset(BenevaloQuerySet)


@reversion.register()
class Benevalo(models.Model):
    """
    Les actions de bénévolat sont au cœur du logiciel Bénévalibre. Elles sont
    attachées :

    * à un⋅e bénévole ;
    * à une association ;
    * à un éventuel projet ;
    * à un éventuel niveau de bénévolat ;
    * à une catégorie.

    Elles ont pour attribut un `titre`, une éventuelle `description`, une
    `durée` de bénévolat et/ou la `distance` si l'action de bénévolat implique
    un déplacement.

    Elles comportent également une `date d'exécution`. Dans l'hypothèse où
    l'exécution s'est étalée dans le temps, on peut définir une période
    d'exécution en reseignant une `date de fin d'exécution`.
    """

    class Meta:
        verbose_name = "action de bénévolat"
        verbose_name_plural = "actions de bénévolat"

    objects = BenevaloManager()

    user = models.ForeignKey(
        User, verbose_name="bénévole", on_delete=models.CASCADE
    )
    association = models.ForeignKey(
        'association.Association',
        verbose_name="association",
        on_delete=models.SET_NULL,
        null=True,
    )
    project = models.ForeignKey(
        'association.Project',
        verbose_name="projet",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
    )
    level = models.ForeignKey(
        'association.Level',
        verbose_name="niveau",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
    )
    category = models.ForeignKey(
        'association.Category',
        verbose_name="catégorie",
        on_delete=models.SET_NULL,
        null=True,
    )
    execution_date = models.DateField("date", default=date.today)
    execution_end_date = models.DateField("date de fin", blank=True, null=True)
    title = models.CharField(
        "titre", help_text="Une courte description", max_length=127
    )
    description = models.TextField("description", blank=True, null=True)
    distance = models.DecimalField(
        "distance",
        help_text="La distance parcourue en kilomètres.",
        max_digits=7,
        decimal_places=2,
        null=True,
        blank=True,
    )
    duration = models.DurationField(
        "durée",
        help_text="Le temps consacré à cette action de bénévolat.",
        null=True,
        blank=True,
    )

    is_active = models.BooleanField(default=True, editable=False)

    # FIXME in near future
    # archive,
    # misc parameters:
    # - publication params

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)

    def clean(self):
        errors = []

        if not (self.duration or self.distance):
            errors.append(
                ValidationError(
                    "Une saisie de bénévolat doit posséder à minima une durée "
                    "ou une distance."
                )
            )
        if self.association_id:
            if (
                self.project_id
                and self.association_id != self.project.association_id
            ):
                errors.append(
                    ValidationError(
                        "Le projet sélectionné n'est pas un projet de "
                        "l'association. Veuillez choisir un projet valide."
                    )
                )
            if (
                self.level_id
                and self.association_id != self.level.association_id
            ):
                errors.append(
                    ValidationError(
                        "Le niveau sélectionné n'est pas un niveau de "
                        "l'association. Veuillez choisir un niveau valide."
                    )
                )
            if (
                self.category_id
                and self.association_id != self.category.association_id
            ):
                errors.append(
                    ValidationError(
                        "La catégorie sélectionnée n'est pas une catégorie de "
                        "l'association. Veuillez choisir une catégorie valide."
                    )
                )

            if (
                self.user_id
                and not self.association.engagement_set.active()
                .filter(user_id=self.user_id)
                .exists()
            ):
                errors.append(
                    ValidationError(
                        "L'utilisat⋅eur⋅rice sélectionné n'est pas un⋅e "
                        "bénévole de l'association. Veuillez choisir un⋅e "
                        "utilisat⋅eur⋅rice valide."
                    )
                )

        if errors:
            raise ValidationError(errors)

    def __str__(self):
        return self.title
