import datetime

from django import forms
from django.core import validators

zero_timedelta = datetime.timedelta()


class SplitDurationWidget(forms.MultiWidget):
    """
    A widget that splits duration input into two <input type="number"> boxes,
    one for the hours and another for the minutes.
    """

    template_name = 'forms/widgets/splitduration.html'

    def __init__(self, attrs=None, hours_attrs=None, minutes_attrs=None):
        widgets = (
            forms.NumberInput(
                attrs=attrs if hours_attrs is None else hours_attrs
            ),
            forms.NumberInput(
                attrs=attrs if minutes_attrs is None else minutes_attrs
            ),
        )
        super().__init__(widgets)

    def decompress(self, value):
        if value:
            hours = value / datetime.timedelta(hours=1)
            minutes = (hours % 1) * 60
            return [int(hours), int(minutes)]
        return [None, None]


class SplitDurationField(forms.MultiValueField):
    """
    A Field that splits duration into hours and minutes.

    By default, the duration must be greater or equal to zero and one of the
    field can be left empty. It is possible to modify that with `min_value`
    and/or `max_value` - which must be `datetime.timedelta` objects - and
    `require_all_fields` arguments.
    """

    widget = SplitDurationWidget
    default_error_messages = {
        'required': "Un nombre d'heures et/ou de minutes est requis.",
        'invalid_hours': "Veuillez saisir un nombre d'heures valide.",
        'invalid_minutes': "Veuillez saisir un nombre de minutes valide.",
    }

    def __init__(self, *, max_value=None, min_value=zero_timedelta, **kwargs):
        self.max_value, self.min_value = max_value, min_value

        errors = self.default_error_messages.copy()
        if 'error_messages' in kwargs:
            errors.update(kwargs['error_messages'])
        fields = (
            forms.IntegerField(
                error_messages={'invalid': errors['invalid_hours']}
            ),
            forms.IntegerField(
                error_messages={'invalid': errors['invalid_minutes']}
            ),
        )
        # One of the field can be empty by default.
        kwargs.setdefault('require_all_fields', False)
        super().__init__(fields, **kwargs)

        # Set 'required' to False on the individual fields if they are not all
        # required, because the required validation will be handled by
        # MultiValueField.clean().
        if not self.require_all_fields:
            for f in fields:
                f.required = False

        if max_value is not None:
            self.validators.append(validators.MaxValueValidator(max_value))
        if min_value is not None:
            self.validators.append(validators.MinValueValidator(min_value))

    def compress(self, data_list):
        if data_list:
            return datetime.timedelta(
                hours=data_list[0] or 0, minutes=data_list[1] or 0
            )
        return None

    def widget_attrs(self, widget):
        attrs = super().widget_attrs(widget)
        if isinstance(widget, SplitDurationWidget):
            # Set attributes directly to the subwidgets as it is not easy at
            # this time to specify them separately.
            if self.min_value is not None:
                widget.widgets[0].attrs['min'] = int(
                    self.min_value / datetime.timedelta(hours=1)
                )
                widget.widgets[1].attrs['min'] = (
                    # Do not set a fine minimum for positive values.
                    int(self.min_value / datetime.timedelta(minutes=1))
                    if self.min_value < zero_timedelta
                    else 0
                )
            if self.max_value is not None:
                widget.widgets[0].attrs['max'] = int(
                    self.max_value / datetime.timedelta(hours=1)
                )
                widget.widgets[1].attrs['max'] = int(
                    self.max_value / datetime.timedelta(minutes=1)
                )
        return attrs
